==> isolate_manager-4.2.1.tar.gz: test/isolate_manager_test.dart <==
import 'dart:async';
import 'dart:convert';

import 'package:isolate_manager/isolate_manager.dart';
import 'package:isolate_manager/src/isolate_worker/isolate_worker_stub.dart'
    as isolate_worker_stub;
import 'package:test/test.dart';

//  dart test
//  dart test --platform=chrome,vm

void main() {
  group('Test funtions -', () {
    test('isolateWorker stub', () {
      expect(
        () => isolate_worker_stub.isolateWorker((message) {}),
        throwsUnimplementedError,
      );
    });
  });

  test('Test IsolateManager.create: Basic Usage', () async {
    // Create IsolateContactor
    print('Create IsolateManager instance');
    final isolateManager = IsolateManager.create(
      fibonacci,
      concurrent: 4,
    );

    print('Computing IsolateManager instance...');
    final result = await isolateManager.compute(3);

    expect(result, fibonacci(3));

    print('Dispose IsolateManager instance.');
    isolateManager.stop();
  });

  test('Test IsolateManager.create', () async {
    // Create IsolateContactor
    print('Create IsolateManager instance');
    final isolateManager = IsolateManager.create(
      fibonacci,
      concurrent: 4,
    );

    expect(isolateManager.isStarted, equals(false));

    print('Starting IsolateManager instance...');
    await isolateManager.start();

    expect(isolateManager.isStarted, equals(true));
    await isolateManager.ensureStarted;

    expect(isolateManager.queuesLength, equals(0));

    print('Computing IsolateManager instance...');
    await Future.wait([
      for (int i = 0; i < 10; i++)
        isolateManager.compute(i).then((value) {
          print('Fibonacci $i = $value');
          expect(value, fibonacci(i));
        })
    ]);

    await Future.delayed(Duration(seconds: 3));

    print('Dispose IsolateManager instance.');
    isolateManager.stop();
  });

  test('Test IsolateManager.createOwnIsolate', () async {
    // Create IsolateContactor
    print('Create IsolateManager instance');
    final isolateManager = IsolateManager<int, int>.createOwnIsolate(
      isolateFunction,
      concurrent: 4,
      initialParams: ['Test initialParams 0', 'Test initialParams 1'],
    )..start();

    isolateManager.stream.listen((value) {
      print('Stream: $value');
    })
        // Do not need to catch the error here
        .onError((error) {});

    print('Computing IsolateManager instance...');
    await Future.wait([
      for (int i = 0; i < 10; i++)
        isolateManager.compute(i).then((value) {
          print('Fibonacci $i = $value');
          expect(value, fibonacci(i));
        })
    ]);
    await Future.delayed(Duration(seconds: 3));

    print('Restarting IsolateManager instance...');
    await isolateManager.restart();

    print('Computing IsolateManager instance second times...');
    await Future.wait([
      for (int i = 5; i < 13; i++)
        isolateManager.compute(i).then((value) {
          print('Fibonacci $i = $value');
          expect(value, fibonacci(i));
        })
    ]);

    print('Exception');
    expect(() => isolateManager.sendMessage(-1), throwsStateError);

    await Future.delayed(Duration(seconds: 3));

    print('Dispose IsolateManager instance.');
    await isolateManager.stop();
  });

  test('Test IsolateManager.create with Worker', () async {
    // Create IsolateContactor
    print('Create IsolateManager instance');
    final isolateManager = IsolateManager.create(
      fibonacci,
      workerName: 'fibonacci',
      concurrent: 4,
    );

    print('Starting IsolateManager instance...');
    await isolateManager.start();

    print('Computing IsolateManager instance...');
    await Future.wait([
      for (int i = 0; i < 10; i++)
        isolateManager.compute(i).then((value) {
          final realFib = fibonacci(i);
          print('Fibonacci $i = $value == $realFib');

          expect(value, realFib);
        })
    ]);

    await Future.delayed(Duration(seconds: 3));

    print('Dispose IsolateManager instance.');
    isolateManager.stop();
  });

  test('Test with Exception function', () async {
    final isolateManager = IsolateManager.create(
      errorFunction,
      concurrent: 1,
      isDebug: true,
    );
    await isolateManager.start();

    expect(
      () async => await isolateManager.compute([50, 50]),
      throwsStateError,
    );

    await Future.delayed(Duration(seconds: 3));

    await isolateManager.stop();
  });

  test('Test with Exception function with available callback', () async {
    final isolateManager = IsolateManager.create(
      errorFunction,
      concurrent: 1,
      isDebug: true,
    );
    await isolateManager.start();

    expect(
      () => isolateManager.compute([50, 50], callback: (value) {
        return true;
      }),
      throwsStateError,
    );

    await Future.delayed(Duration(seconds: 3));

    await isolateManager.stop();
  });

  test('Test with Exception function with eagerError is true', () async {
    final isolateManager = IsolateManager.create(
      errorFunction,
      concurrent: 2,
      isDebug: true,
    );
    await isolateManager.start();
    final List<Future> futures = [];

    for (var i = 0; i < 100; i++) {
      futures.add(isolateManager.compute([i, 20]));
    }

    expect(
      () async => await Future.wait(futures, eagerError: true),
      throwsStateError,
    );

    await Future.delayed(Duration(seconds: 3));

    await isolateManager.stop();
  });

  test(
      'Test with Exception function with eagerError is true with available callback',
      () async {
    final isolateManager = IsolateManager.create(
      errorFunction,
      concurrent: 2,
      isDebug: true,
    );
    await isolateManager.start();
    final List<Future> futures = [];

    for (var i = 0; i < 100; i++) {
      futures.add(isolateManager.compute([i, 20], callback: (value) => true));
    }

    expect(
      () async => await Future.wait(futures, eagerError: true),
      throwsStateError,
    );

    await Future.delayed(Duration(seconds: 3));

    await isolateManager.stop();
  });

  test('Test with Exception function with eagerError is false', () async {
    final isolateManager = IsolateManager.create(
      errorFunction,
      concurrent: 2,
      isDebug: true,
    );
    await isolateManager.start();
    final List<Future> futures = [];

    for (var i = 0; i < 100; i++) {
      futures.add(isolateManager.compute([i, 20]));
    }

    expect(
      () async => await Future.wait(futures, eagerError: false),
      throwsStateError,
    );

    await Future.delayed(Duration(seconds: 3));

    await isolateManager.stop();
  });

  test(
      'Test with Exception function with eagerError is false with available callback',
      () async {
    final isolateManager = IsolateManager.create(
      errorFunction,
      concurrent: 2,
      isDebug: true,
    );
    await isolateManager.start();
    final List<Future> futures = [];

    for (var i = 0; i < 100; i++) {
      futures.add(isolateManager.compute([i, 20], callback: (value) => true));
    }

    expect(
      () async => await Future.wait(futures, eagerError: false),
      throwsStateError,
    );

    await Future.delayed(Duration(seconds: 3));

    await isolateManager.stop();
  });

  test('Test with IsolateCallback', () async {
    final isolateManager = IsolateManager<String, int>.createOwnIsolate(
      isolateCallbackFunction,
      concurrent: 1,
      isDebug: true,
    );
    await isolateManager.start();

    final result = await isolateManager.compute(1, callback: (value) {
      final decoded = jsonDecode(value) as Map;
      // Do not return this [value] as the final result
      if (decoded.containsKey('source')) {
        return false;
      }

      // Return this [value] as the final result
      return true;
    });

    final decoded = jsonDecode(result) as Map;
    expect(
      decoded.containsKey('data'),
      equals(true),
    );

    await isolateManager.stop();
  });
}

@pragma('vm:entry-point')
int fibonacci(int n) {
  if (n < 0) throw StateError('n<0');
  if (n == 0) return 0;
  if (n <= 2) return 1;

  double n1 = 0, n2 = 1, n3 = 1;

  for (int i = 2; i <= n; i++) {
    n3 = n1 + n2;
    n1 = n2;
    n2 = n3;
  }

  return n3.round();
}

@pragma('vm:entry-point')
void isolateFunction(dynamic params) {
  late IsolateManagerController controller;
  controller = IsolateManagerController<int, int>(
    params,
    onDispose: () {
      controller.close();
    },
  );

  final initialParams = controller.initialParams;
  print('initialParams 0: ${initialParams[0]}');
  print('initialParams 1: ${initialParams[1]}');

  controller.onIsolateMessage.listen((message) {
    try {
      final result = fibonacci(message);
      controller.sendResult(result);
    } catch (err, stack) {
      controller.sendResultError(IsolateException(err, stack));
    }
  });
}

@pragma('vm:entry-point')
void isolateCallbackFunction(dynamic params) {
  final controller = IsolateManagerController<String, int>(params);

  controller.onIsolateMessage.listen((message) {
    try {
      for (int i = 0; i < 10; i++) {
        controller.sendResult(jsonEncode({'source': '$i'}));
      }

      controller.sendResult(jsonEncode({'data': 'data'}));
    } catch (err, stack) {
      controller.sendResultError(IsolateException(err, stack));
    }
  });
}

@pragma('vm:entry-point')
int errorFunction(List<int> value) {
  if (value[0] == 50) {
    return throw StateError('The exception is threw at value[0] = ${value[0]}');
  }
  return value[0] + value[1];
}

==> isolate_manager-4.2.2.tar.gz: test/isolate_manager_test.dart <==
import 'dart:async';
import 'dart:convert';

import 'package:isolate_manager/isolate_manager.dart';
import 'package:test/test.dart';

//  dart test
//  dart test --platform=chrome,vm

void main() {
  group('Test functions -', () {
    test('IsolateFunctionHelper.isolateWorker', () {
      try {
        IsolateFunctionHelper.workerFunction((message) => {});
      } catch (e) {
        expect(e, isA<UnimplementedError>());
      }
    });
  });

  test('Test IsolateManager.create: Basic Usage', () async {
    // Create IsolateContactor
    final isolateManager = IsolateManager.create(
      fibonacci,
      concurrent: 4,
    );

    final result = await isolateManager.compute(3);

    expect(result, fibonacci(3));

    isolateManager.stop();
  });

  test('Test IsolateManager.create', () async {
    // Create IsolateContactor
    final isolateManager = IsolateManager.create(
      fibonacci,
      concurrent: 4,
    );

    expect(isolateManager.isStarted, equals(false));

    await isolateManager.start();

    expect(isolateManager.isStarted, equals(true));
    await isolateManager.ensureStarted;

    expect(isolateManager.queuesLength, equals(0));

    await Future.wait([
      for (int i = 0; i < 10; i++)
        isolateManager.compute(i).then((value) {
          expect(value, fibonacci(i));
        })
    ]);

    await Future.delayed(Duration(seconds: 3));

    isolateManager.stop();
  });

  test('Test IsolateManager.createCustom', () async {
    // Create IsolateContactor
    final isolateManager = IsolateManager<int, int>.createCustom(
      isolateFunction,
      concurrent: 4,
      initialParams: ['Test initialParams 0', 'Test initialParams 1'],
    )..start();

    isolateManager.stream
        .listen((value) {})
        // Do not need to catch the error here
        .onError((error) {});

    await Future.wait([
      for (int i = 0; i < 10; i++)
        isolateManager.compute(i).then((value) {
          expect(value, fibonacci(i));
        })
    ]);
    await Future.delayed(Duration(seconds: 3));

    await isolateManager.restart();

    await Future.wait([
      for (int i = 5; i < 13; i++)
        isolateManager.compute(i).then((value) {
          expect(value, fibonacci(i));
        })
    ]);

    expect(() => isolateManager.sendMessage(-1), throwsStateError);

    await Future.delayed(Duration(seconds: 3));

    await isolateManager.stop();
  });

  test('Test IsolateManager.createCustom with automatically handlers',
      () async {
    // Create IsolateContactor
    final isolateManager = IsolateManager<int, int>.createCustom(
      isolateFunctionWithAutomaticallyHandlers,
      concurrent: 4,
      initialParams: ['Test initialParams 0', 'Test initialParams 1'],
    )..start();

    isolateManager.stream
        .listen((value) {})
        // Do not need to catch the error here
        .onError((error) {});

    await Future.wait([
      for (int i = 0; i < 10; i++)
        isolateManager.compute(i).then((value) {
          expect(value, fibonacci(i));
        })
    ]);
    await Future.delayed(Duration(seconds: 3));

    await isolateManager.restart();

    await Future.wait([
      for (int i = 5; i < 13; i++)
        isolateManager.compute(i).then((value) {
          expect(value, fibonacci(i));
        })
    ]);

    expect(() => isolateManager.sendMessage(-1), throwsStateError);

    await Future.delayed(Duration(seconds: 3));

    await isolateManager.stop();
  });

  test('Test IsolateManager.create with Worker', () async {
    // Create IsolateContactor
    final isolateManager = IsolateManager.create(
      fibonacci,
      workerName: 'fibonacci',
      concurrent: 4,
    );

    await isolateManager.start();

    await Future.wait([
      for (int i = 0; i < 10; i++)
        isolateManager.compute(i).then((value) {
          final realFib = fibonacci(i);

          expect(value, realFib);
        })
    ]);

    await Future.delayed(Duration(seconds: 3));

    isolateManager.stop();
  });

  test('Test with Exception future function', () async {
    final isolateManager = IsolateManager.create(
      errorFunctionFuture,
      concurrent: 1,
    );
    await isolateManager.start();

    expect(
      () async => await isolateManager.compute([50, 50]),
      throwsStateError,
    );

    await Future.delayed(Duration(seconds: 3));

    await isolateManager.stop();
  });

  test('Test with Exception function', () async {
    final isolateManager = IsolateManager.create(
      errorFunction,
      concurrent: 1,
    );
    await isolateManager.start();

    expect(
      () async => await isolateManager.compute([50, 50]),
      throwsStateError,
    );

    await Future.delayed(Duration(seconds: 3));

    await isolateManager.stop();
  });

  test('Test with Exception function with available callback', () async {
    final isolateManager = IsolateManager.create(
      errorFunction,
      concurrent: 1,
    );
    await isolateManager.start();

    expect(
      () => isolateManager.compute([50, 50], callback: (value) {
        return true;
      }),
      throwsStateError,
    );

    await Future.delayed(Duration(seconds: 3));

    await isolateManager.stop();
  });

  test('Test with Exception function with eagerError is true', () async {
    final isolateManager = IsolateManager.create(
      errorFunction,
      concurrent: 2,
    );
    await isolateManager.start();
    final List<Future> futures = [];

    for (var i = 0; i < 100; i++) {
      futures.add(isolateManager.compute([i, 20]));
    }

    expect(
      () async => await Future.wait(futures, eagerError: true),
      throwsStateError,
    );

    await Future.delayed(Duration(seconds: 3));

    await isolateManager.stop();
  });

  test(
      'Test with Exception function with eagerError is true with available callback',
      () async {
    final isolateManager = IsolateManager.create(
      errorFunction,
      concurrent: 2,
    );
    await isolateManager.start();
    final List<Future> futures = [];

    for (var i = 0; i < 100; i++) {
      futures.add(isolateManager.compute([i, 20], callback: (value) => true));
    }

    expect(
      () async => await Future.wait(futures, eagerError: true),
      throwsStateError,
    );

    await Future.delayed(Duration(seconds: 3));

    await isolateManager.stop();
  });

  test('Test with Exception function with eagerError is false', () async {
    final isolateManager = IsolateManager.create(
      errorFunction,
      concurrent: 2,
    );
    await isolateManager.start();
    final List<Future> futures = [];

    for (var i = 0; i < 100; i++) {
      futures.add(isolateManager.compute([i, 20]));
    }

    expect(
      () async => await Future.wait(futures, eagerError: false),
      throwsStateError,
    );

    await Future.delayed(Duration(seconds: 3));

    await isolateManager.stop();
  });

  test(
      'Test with Exception function with eagerError is false with available callback',
      () async {
    final isolateManager = IsolateManager.create(
      errorFunction,
      concurrent: 2,
    );
    await isolateManager.start();
    final List<Future> futures = [];

    for (var i = 0; i < 100; i++) {
      futures.add(isolateManager.compute([i, 20], callback: (value) => true));
    }

    expect(
      () async => await Future.wait(futures, eagerError: false),
      throwsStateError,
    );

    await Future.delayed(Duration(seconds: 3));

    await isolateManager.stop();
  });

  test('Test with IsolateCallback', () async {
    final isolateManager = IsolateManager<String, int>.createCustom(
      isolateCallbackFunction,
      concurrent: 1,
    );
    await isolateManager.start();

    final result = await isolateManager.compute(1, callback: (value) {
      final decoded = jsonDecode(value) as Map;
      // Do not return this [value] as the final result
      if (decoded.containsKey('source')) {
        return false;
      }

      // Return this [value] as the final result
      return true;
    });

    final decoded = jsonDecode(result) as Map;
    expect(
      decoded.containsKey('data'),
      equals(true),
    );

    await isolateManager.stop();
  });
}

@pragma('vm:entry-point')
int fibonacci(int n) {
  if (n < 0) throw StateError('n<0');
  if (n == 0) return 0;
  if (n <= 2) return 1;

  double n1 = 0, n2 = 1, n3 = 1;

  for (int i = 2; i <= n; i++) {
    n3 = n1 + n2;
    n1 = n2;
    n2 = n3;
  }

  return n3.round();
}

@pragma('vm:entry-point')
void isolateFunction(dynamic params) {
  IsolateFunctionHelper.customFunction<int, int>(
    params,
    onEvent: (controller, message) {
      try {
        final result = fibonacci(message);
        controller.sendResult(result);
      } catch (err, stack) {
        controller.sendResultError(IsolateException(err, stack));
      }
      return 0;
    },
    onInitial: (controller, initialParams) {},
    onDispose: (controller) {},
    autoHandleException: false,
    autoHandleResult: false,
  );
}

@pragma('vm:entry-point')
void isolateFunctionWithAutomaticallyHandlers(dynamic params) {
  IsolateFunctionHelper.customFunction<int, int>(
    params,
    onEvent: (controller, message) {
      return fibonacci(message);
    },
    onInitial: (controller, initialParams) {},
    onDispose: (controller) {},
    autoHandleException: true,
    autoHandleResult: true,
  );
}

@pragma('vm:entry-point')
void isolateCallbackFunction(dynamic params) {
  IsolateFunctionHelper.customFunction(
    params,
    onEvent: (controller, message) {
      try {
        for (int i = 0; i < 10; i++) {
          controller.sendResult(jsonEncode({'source': '$i'}));
        }

        controller.sendResult(jsonEncode({'data': 'data'}));
      } catch (err, stack) {
        controller.sendResultError(IsolateException(err, stack));
      }
    },
    autoHandleException: false,
    autoHandleResult: false,
  );
}

@pragma('vm:entry-point')
int errorFunction(List<int> value) {
  if (value[0] == 50) {
    return throw StateError('The exception is threw at value[0] = ${value[0]}');
  }
  return value[0] + value[1];
}

@pragma('vm:entry-point')
Future<int> errorFunctionFuture(List<int> value) async {
  await Future.delayed(Duration(seconds: 1));

  if (value[0] == 50) {
    return throw StateError('The exception is threw at value[0] = ${value[0]}');
  }
  return value[0] + value[1];
}
